@@ -1,10 +1,17 @@
 timeout_set 1 hour
 
-CHUNKSERVERS=2 \
+CHUNKSERVERS=3 \
 	MOUNT_EXTRA_CONFIG="mfscachemode=NEVER" \
 	setup_local_empty_lizardfs info
 
 cd ${info[mount0]}
+directories=()
+for goal in 1 2 xor2; do
+	dir="goal_$goal"
+	mkdir "$dir"
+	lizardfs setgoal "$goal" "$dir"
+	directories+=("$dir")
+done
 
 test_worker() {
 	local min_size=$1
@@ -14,16 +21,17 @@ test_worker() {
 	while ! test_frozen; do
 		local block_size=$(pseudorandom 1024 128K)
 		local file_size=$(pseudorandom $min_size $max_size)
-		local file=$(unique_file)
+		local dirindex=$(pseudorandom 0 $((${#directories[@]} - 1)))
+		local file="${directories[dirindex]}/$(unique_file)"
 		echo FILE_SIZE=$file_size BLOCK_SIZE=$block_size file-generate "$file"
 		if FILE_SIZE=$file_size BLOCK_SIZE=$block_size file-generate "$file"; then
 			if ! file-validate "$file"; then
-				test_add_failure "Invalid data: block $block_size, size $file_size"
+				test_add_failure "Invalid data: block $block_size, size $file_size, file $file"
 			fi
-			mfssettrashtime 0 "$file" &>/dev/null
+			lizardfs settrashtime 0 "$file" &>/dev/null
 			rm -f "$file"
 		else
-			test_add_failure "file-validate failed: block $block_size, size $file_size"
+			test_add_failure "file-validate failed: block $block_size, size $file_size, file $file"
 		fi
 	done
 }
